% PAIRWISELEARN Pairwise CRF learning by cross-validation
%
% This is a reference implementation for the synthetic experiments on lower
% linear enevelope inference and learning described in
%
%    "Max-margin Learning for Lower Linear Envelope Potentials in Binary
%    Markov Random Fields", Stephen Gould, ICML 2011.
%
% Copyright (c) 2011, Stephen Gould <stephen.gould@anu.edu.au>
% All rights reserved.
%

function [params, history] = pairwiseLearn(x, y, options);

% initialize parameters
params.unaryWeight = 1.0;
params.pairwiseWeight = 0.0;
params.linEnvCoeffs = [];

% cross-validate pairwise term
y_hat = linEnvInf(x, 0.0, []);
history = struct('params', params, 'y_hat', y_hat);

bestLoss = sum(y_hat(:) ~= y(:));
for p = linspace(0.0, 1.0, options.maxIters),
    y_hat = linEnvInf(x, p, []);
    loss = sum(y_hat(:) ~= y(:));
    if (loss < bestLoss),
        bestLoss = loss;
        params.pairwiseWeight = p;

        if (options.figWnd > 0),
            figure(options.figWnd);
            subplot(1, 2, 1); colormap(gray); image(255 * y_hat);
            title(['best inferred labeling (p=', num2str(p), ')']);
            drawnow;
        end;
        
        history(end + 1) = struct('params', params, 'y_hat', y_hat);
    end;

    % show results
    if (options.figWnd > 0),
        figure(options.figWnd);
        subplot(1, 2, 2); colormap(gray); image(255 * y_hat);
        title(['inferred labeling with p=', num2str(p)]);
        drawnow;
    end;
end;

